package com.dodo.leancloud

import android.app.Activity
import android.util.Log
import com.avos.avoscloud.*
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import com.avos.avoscloud.AVException
import com.avos.avoscloud.AVCloudQueryResult
import com.avos.avoscloud.CloudQueryCallback
import com.avos.avoscloud.AVQuery
import com.dodo.leancloud.AVObjectPlugin.parseFormAVObject
import java.util.ArrayList


class LeancloudPlugin private constructor(val activity: Activity) : MethodCallHandler {
    companion object {
        val TAG = LeancloudPlugin::class.java.simpleName

        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val activity = registrar.activity()
            val channel = MethodChannel(registrar.messenger(), "leancloud")
            channel.setMethodCallHandler(LeancloudPlugin(activity))
        }
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "initialize" -> {
                Log.e(TAG, "initialize")
                val appID = call.argument<String>("appID")
                val appKey = call.argument<String>("appKey")
                AVOSCloud.initialize(activity.applicationContext, appID, appKey)
                AVOSCloud.setDebugLogEnabled(BuildConfig.DEBUG)
                result.success(null)
            }
            "signUp" -> {
                Log.e(TAG, "signUp")
                val username = call.argument<String>("username")
                val password = call.argument<String>("password")
                val user = AVUser()// 新建 AVUser 对象实例
                user.username = username// 设置用户名
                user.setPassword(password)// 设置密码
                user.signUpInBackground(object : SignUpCallback() {
                    override fun done(e: AVException?) {
                        if (e == null) {
                            result.success(true)
                        } else {
                            // 失败的原因可能有多种，常见的是用户名已经存在。
                            Log.e(TAG, e.message, e)
                            result.success(false)
                        }
                    }
                })
            }
            "login" -> {
                Log.e(TAG, "login")
                val username = call.argument<String>("username")
                val password = call.argument<String>("password")
                AVUser.logInInBackground(username, password, object : LogInCallback<AVUser>() {
                    override fun done(avUser: AVUser?, e: AVException?) {
                        if (e == null && avUser != null) {
                            avUser.getRolesInBackground(object : AVCallback<List<AVRole>>() {
                                override fun internalDone0(t: List<AVRole>?, avException: AVException?) {
                                    if (t == null) {
                                        result.success(null)
                                        return
                                    }
                                    val list = mutableListOf<String>()
                                    for (role in t) {
                                        list.add(role.name)
                                    }
                                    result.success(list)
                                }

                            })
                        } else {
                            Log.e(TAG, e?.message, e)
                            result.success(null)
                        }
                    }
                })
            }
            "logout" -> {
                Log.e(TAG, "logout")
                AVUser.logOut()
                result.success(null)
            }
            "putAVObject" -> {
                Log.e(TAG, "putAVObject")
                val json = call.argument<String>("json")
                Log.e(TAG, "json arg: $json")

                val avObjectPlugin = AVObjectPlugin.parseFormJson(json)
                Log.e(TAG, avObjectPlugin.toString())

                avObjectPlugin.saveInBackground {
                    Log.e(TAG, "put done $it")

                }
            }
            "getAVObject" -> {
                Log.e(TAG, "getAVObject")
                val json = call.argument<String>("json")
                Log.e(TAG, "json arg: $json")

                val avObjectPlugin = AVObjectPlugin.parseFormJson(json)
                Log.e(TAG, avObjectPlugin.toString())

                avObjectPlugin.findInBackground {
                    Log.e(TAG, "get done $it")
                    result.success(it)
                }
            }
            "deleteAVObject" -> {
                Log.e(TAG, "deleteAVObject")
                val json = call.argument<String>("json")
                Log.e(TAG, "json arg: $json")

                val avObjectPlugin = AVObjectPlugin.parseFormJson(json)
                Log.e(TAG, avObjectPlugin.toString())

                avObjectPlugin.deleteInBackground {
                    Log.e(TAG, "get done $it")
                    result.success(it)
                }
            }
            "doCQLQuery" -> {
                Log.e(TAG, "doCQLQuery")
                val json = call.argument<String>("json")
                val cql = call.argument<String>("cql")
                Log.e(TAG, "json arg: $json")
                Log.e(TAG, "cql arg: $cql")

                val avObjectPlugin = AVObjectPlugin.parseFormJson(json)
                Log.e(TAG, avObjectPlugin.toString())

                avObjectPlugin.doCloudQueryInBackground(cql) { jsons ->
                    Log.e(TAG, "get done $jsons")
                    result.success(jsons)
                }
            }
            else -> {
                result.notImplemented()
            }
        }
    }
}
