import 'dart:io';

import 'package:flutter/material.dart';
import 'package:leancloud/avfile.dart';
import 'package:leancloud/avobject.dart';
import 'package:flutter/services.dart';
import 'package:leancloud/leancloud.dart';
import 'package:image_picker/image_picker.dart';
import 'test_bean.dart';

class AVObjectPage extends StatefulWidget {
  final List<String> _roles;

  AVObjectPage(this._roles, {Key key}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _AVObjectState(_roles);
}

class _AVObjectState extends State<AVObjectPage> {
  List<String> _roles;
  String _result = "";
  AVObject currentAVObject;

  TextEditingController _indexController = TextEditingController();

  TextEditingController _wordsController = TextEditingController();
  TextInputFormatter _numberInputFormatter = NumberInputFormatter();
  List<TextInputFormatter> _inputFormatters = List();

  _AVObjectState(this._roles) {
    _inputFormatters.add(_numberInputFormatter);
  }

  AVFile avFile = AVFile();

  Future getImage() async {
    var image = await ImagePicker.pickImage(source: ImageSource.gallery);
    if(image==null){
      return;
    }
    setState(() {
      var path = image.path;
      var name = path.substring(path.lastIndexOf("/")+1);
      avFile = AVFile.name(name);
      avFile.localPath = path;
    });
  }

  dynamic getImageView() {
    if (avFile.localPath != null && avFile.localPath.isNotEmpty) {
      return Image.file(
        File(avFile.localPath),
        width: 100,
        height: 100,
        fit: BoxFit.cover,
      );
    } else if (avFile.url != null && avFile.url.isNotEmpty) {
      return Image.network(
        avFile.url,
        width: 100,
        height: 100,
        fit: BoxFit.cover,
      );
    } else {
      return Offstage(
        offstage: _roles.length == 0,
        child: RaisedButton(
            child: Text("选择图片"),
            textColor: Colors.blue,
            onPressed: getImage),
      );
    }
  }

  _put() async {
    bool result = false;
    var _index = int.parse(_indexController.text);
    try {
      var testBean = TestBean();
      testBean.index = _index;
      testBean.words = _wordsController.text;

      testBean.file = avFile;

      var avObject = testBean.toAVObject();

      result = await Leancloud.putAVObject(avObject);
    } on PlatformException catch (e, s) {
      print('Exception details:\n $e');
      print('Stack trace:\n $s');
    }

    setState(() {
      if (result) {
        _result = "添加成功";
      } else {
        _result = "添加失败";
      }
    });
  }

  _get() async {
    List<AVObject> result;
    try {
      var testBean = TestBean();
      testBean.index = int.parse(_indexController.text);

      var avObject = testBean.toAVObject();

      result = await Leancloud.getAVObject(avObject);
    } on PlatformException catch (e, s) {
      print('Exception details:\n $e');
      print('Stack trace:\n $s');
    }

    setState(() {
      if (result != null && result.length != 0) {
        _result = "查询成功";
        currentAVObject = result[0];
        TestBean resultBean  = TestBean().fromAVObject(currentAVObject);
        avFile = resultBean.file;
        _wordsController.text = resultBean.words;
      } else {
        avFile = AVFile();
        _result = "查询失败";
      }
    });
  }

  _CQL() async {
    avFile.localPath = null;
    List<AVObject> result;
    try {
      var testBean = TestBean();
      var avObject = testBean.toAVObject();
      String cql =
          "select * from ${TestBean.CLASSNAME} where ${TestBean.INDEX} = 1";
//      String cql = "update ${TestBean.CLASSNAME} set ${TestBean.WORDS}='我呦呦呦更新了第一行数据' where ${AVObject.OBJECT_ID} = '${currentAVObject.getString(AVObject.OBJECT_ID)}'";

      result = await Leancloud.doCQLQuery(avObject, cql);
    } on PlatformException catch (e, s) {
      print('Exception details:\n $e');
      print('Stack trace:\n $s');
    }

    setState(() {
      if (result != null && result.length != 0) {
        _result = "查询成功";
        currentAVObject = result[0];
        _wordsController.text = currentAVObject.getString('words');
      } else {
        _result = "查询失败";
      }
    });
  }

  _delete() async {
    bool result;
    if (currentAVObject == null) {
      result = false;
    } else {
      try {
        result = await Leancloud.deleteAVObject(currentAVObject);
      } on PlatformException catch (e, s) {
        print('Exception details:\n $e');
        print('Stack trace:\n $s');
      }
    }

    setState(() {
      if (result) {
        _result = "删除成功";
      } else {
        _result = "删除失败";
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("AVObject"),
      ),
      body: Column(
//        mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            TextField(
              inputFormatters: _inputFormatters,
              controller: _indexController,
              autofocus: true,
              decoration: InputDecoration(
                labelText: "index",
                hintText: "请输入index",
              ),
            ),
            TextField(
              controller: _wordsController,
              decoration: InputDecoration(
                labelText: "words",
                hintText: "请输入words",
              ),
            ),
            getImageView(),
            Text('$_result'),
            Offstage(
              offstage: _roles.length == 0,
              child: RaisedButton(
                  child: Text("添加"), textColor: Colors.blue, onPressed: _put),
            ),
            RaisedButton(
                child: Text("查询"), textColor: Colors.blue, onPressed: _get),
            RaisedButton(
                child: Text("cql查询"), textColor: Colors.blue, onPressed: _CQL),
            Offstage(
              offstage: _roles.length == 0,
              child: RaisedButton(
                  child: Text("删除"),
                  textColor: Colors.blue,
                  onPressed: _delete),
            ),
          ]),
    );
  }
}

class NumberInputFormatter extends TextInputFormatter {
  List<String> numbers = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"];

  @override
  TextEditingValue formatEditUpdate(
      TextEditingValue oldValue, TextEditingValue newValue) {
    int selectionIndex;

    var text;
    var oldText = oldValue.text;
    var newText = newValue.text;
    var newSelectionIndex = newValue.selection.end;
    var oldSelectionIndex = oldValue.selection.end;

    if (newText == null || newText.isEmpty) {
      text = newText;
      selectionIndex = newSelectionIndex;
    } else {
      var newChar = newText[newSelectionIndex - 1];
      if (oldText == "0" || !numbers.contains(newChar)) {
        text = oldText;
        selectionIndex = oldSelectionIndex;
      } else {
        text = newText;
        selectionIndex = newSelectionIndex;
      }
    }

    return new TextEditingValue(
      text: text,
      selection: new TextSelection.collapsed(offset: selectionIndex),
    );
  }
}
