import 'dart:convert';
import 'avfile.dart';

class AVObject {
  static const String _OBJECT_PREFIX = 'object_';
  static const String _FILE_PREFIX = 'file_';

  String className;
  Map<String, dynamic> _map = Map();

  AVObject(String theClassName) : className = theClassName;

  AVObject.fromMap(Map<String, dynamic> map){
    if (map == null) {
      return;
    }
    className = map['className'];
    _map = map['_map'];
    for (var key in _map.keys) {
      if (key.startsWith(_OBJECT_PREFIX) || key.startsWith(_FILE_PREFIX)) {
        _map[key] = AVObject.fromMap(_map[key]);
      }
    }
  }

  factory AVObject.fromJson(String jsonStr) {
    if (jsonStr == null || jsonStr.isEmpty) {
      return null;
    }
    var map = json.decode(jsonStr);
    return AVObject.fromMap(map);
  }

  Map<String, dynamic> toMap() =>
      <String, dynamic>{
        'className': className,
        '_map': _map,
      };

  String toJson() {
//    if(className == AVFile.CLASSNAME) {
//      throw "you can not operate AVFILE,put it to other AVObject and operate that object";
//    }
    var map = toMap();
    print(map);
    return json.encode(map);
  }

  void put(final String key, final dynamic value) {
    if (value is AVObject) {
      throw "you need to use putAVObject or putAVFile instead";
    }
    _map[key] = value;
  }

  void putAVObject(final String key, final AVObject value) {
    _map['$_OBJECT_PREFIX$key'] = value;
  }

  void putAVFile(final String key, final AVObject value) {
    _map['$_FILE_PREFIX$key'] = value;
  }

  bool getBool(final String key) {
    return _map[key];
  }

  int getInt(final String key) {
    return _map[key];
  }

  double getDouble(final String key) {
    return _map[key];
  }

  String getString(final String key) {
    return _map[key];
  }

  AVObject getAVObject(final String key) {
    return _map['$_OBJECT_PREFIX$key'];
  }

  AVObject getAVFile(final String key) {
    return _map['$_FILE_PREFIX$key'];
  }

  @override
  String toString() {
    return toJson();
  }
}
